using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace DatabaseProj
{
	/// <summary>
	/// Summary description for ShoppingCart.
	/// </summary>
	public class ShoppingCart : System.Web.UI.Page
	{
		protected System.Web.UI.WebControls.Button ButtonCheckout;
        protected System.Web.UI.WebControls.DataGrid DataGridCDsInCart;
        protected System.Web.UI.WebControls.DataGrid DataGridBooksInCart;
        protected System.Web.UI.WebControls.Label LabelShoppingCart;
        protected System.Web.UI.WebControls.Label lblMusicError;
        protected System.Web.UI.WebControls.Label lblBooksError;
		protected System.Web.UI.WebControls.Button Return;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
            this.ClearErrors();

            if (Page.IsPostBack) return;

            // see what kind of request has been made
            if (this.IsShoppingCartUpdate)
            {
                // add items
                this.AddItemsToShoppingCart();                
            }
            else
            {
                // display what's in the shopping cart(s)
                this.PopulateDataGrids();
            }
		}


        /// <summary>
        /// Clears all error messages from the screen.
        /// </summary>
        private void ClearErrors()
        {
            this.lblMusicError.Visible = false;
            this.lblBooksError.Visible = false;
        }


        /// <summary>
        /// Gets whether there is currently a request to update the shopping cart.
        /// </summary>
        private bool IsShoppingCartUpdate
        {
            get 
            {
                return this.Operation != null && 
                      (this.Operation.ToLower() == "add" 
                    || this.Operation.ToLower() == "set");
            }
        }

        private string Operation
        {
            get { return this.Request["op"]; }
        }


        /// <summary>
        /// Gets the item type that is to be added to the shopping cart.
        /// </summary>
        /// <value>Expected item types are "book" and "cd".</value>
        private string ItemType
        {
            get { return this.Request["type"]; }
        }

        /// <summary>
        /// Gets the productID of the product to be purchased
        /// </summary>
        private int ProductID
        {
            get { return Request["productid"] != null ? int.Parse(Request["productid"]) : 0; }
        }

        /// <summary>
        /// Gets the product dependent ID of the product (ex. ISBN or CDID)
        /// </summary>
        private int ProductDependentID
        {
            get { return Request["uid"] != null ? int.Parse(Request["uid"]) : 0; }
        }


        /// <summary>
        /// Gets the name of the product.
        /// </summary>
        private string ProductName
        {
            get { return Request["name"]; }
        }


        /// <summary>
        /// Gets the number of items to be added or purchased.
        /// </summary>
        private int Quantity
        {
            get { return Request["quantity"] != null ? int.Parse(Request["quantity"]) : 0; }
        }


        /// <summary>
        /// Gets the unit price of the item(s) to be purchased.
        /// </summary>
        private double Price
        {
            get { return Request["price"] != null ? double.Parse(Request["price"]) : 0.0; }
        }


        /// <summary>
        /// Adds items to the shopping cart based on the values of the URL query-string.
        /// </summary>
        private void AddItemsToShoppingCart()
        {
            bool success = false;

            // modify the shopping cart contents according to the query-string arguments
            switch(ItemType.ToLower())
            {
                case "cd":
                    if (this.Operation.ToLower() == "add")
                    {
                        success = this.IncrementCDsPurchased(this.ProductID, this.ProductDependentID, this.ProductName, this.Quantity, this.Price);   
                    }
                    else if (this.Operation.ToLower() == "set")
                    {
                        success = this.AddNewCD(this.ProductID, this.ProductDependentID, this.ProductName, this.Quantity, this.Price);
                    }
                    break;
                case "book":
                    if (this.Operation.ToLower() == "add")
                    {
                        success = this.IncrementBooksPurchased(this.ProductID, this.ProductDependentID, this.ProductName, this.Quantity, this.Price);
                    }
                    else if (this.Operation.ToLower() == "set")
                    {
                        success = this.AddNewBooks(this.ProductID, this.ProductDependentID, this.ProductName, this.Quantity, this.Price);
                    }
                    break;
                default:
                    throw new Exception("Unknown Operation, '" + this.ItemType + "'");
            }

            if (success == false)
                // display an error message
                Response.Write(string.Format("Could not complete purchase: {0} is out of stock", this.ProductName));
            else
                // display the page as the shopping cart information
                Response.Redirect("ShoppingCart.aspx");
        }


        public bool AddNewBooks(int productID, int isbn, string title, int quantity, double unitPrice)
        {
            // get the productID if none is supplied
            if (productID < 0)
                productID = (int)Common.RunStoredProcedure(Common.ConnectionString, "spGetProductIDFromISBN",
                    new SqlParameter("@isbn", isbn)).Tables[0].Rows[0]["ProductID"];

            // get the quantity
            int available = (int)Common.RunStoredProcedure(Common.ConnectionString, "cgillum_inventory.dbo.spGetQuantityInStock", 
                new SqlParameter("@productID", productID)).Tables[0].Rows[0]["Quantity"];

            // check to see if we have this amount in stock
            if (quantity > available)
            {
                return false;
            }

            DataTable bookPurchases = Common.GetBookShoppingCart(this.Session);

            DataRow[] existingProductRows = bookPurchases.Select("ISBN = " + isbn);
            bool createNewRow = (existingProductRows.Length == 0);
            DataRow newBook;
            int toPurchase = 0;
            
            // create a new row or use an existing one (when updating a quantity)
            if (!createNewRow)
            {
                newBook = existingProductRows[0];
                toPurchase = quantity;
            }
            else
            {
                newBook = bookPurchases.NewRow();
                toPurchase = quantity;
            }

            // set the properties
            newBook["ISBN"]      = isbn;
            newBook["BookName"]  = title;
            newBook["quantity"]  = toPurchase;
            newBook["unitPrice"] = unitPrice;

            // add the new row to the table if it's a new row
            if (createNewRow)
                bookPurchases.Rows.Add(newBook);

            return true;
        }

        public bool IncrementBooksPurchased(int productID, int isbn, string title, int quantity, double unitPrice)
        {
            DataTable booksPurchased = Common.GetBookShoppingCart(this.Session);
            int countSoFar  = quantity;
            DataRow[] results = booksPurchased.Select("ISBN = " + isbn);
            if (results.Length > 0)
            {
                countSoFar += (int)results[0]["quantity"];
            }

            return this.AddNewBooks(productID, isbn, title, countSoFar, unitPrice);
        }


        public bool AddNewCD(int productID, int cdID, string cdName, int quantity, double unitPrice)
        {
            // get the productID if none is supplied
            if (productID < 0)
                productID = (int)Common.RunStoredProcedure(Common.ConnectionString, "spGetProductIDFromCDID",
                    new SqlParameter("@cdid", cdID)).Tables[0].Rows[0]["ProductID"];

            // get the quantity
            int available = (int)Common.RunStoredProcedure(Common.ConnectionString, "cgillum_inventory.dbo.spGetQuantityInStock", 
                new SqlParameter("@productID", productID)).Tables[0].Rows[0]["Quantity"];

            // check to see if we have this amount in stock
            if (quantity > available)
            {
                return false;
            }

            DataTable cdPurchases = Common.GetCDShoppingCart(this.Session);

            DataRow[] existingProductRows = cdPurchases.Select("CDID = " + cdID);
            bool createNewRow = (existingProductRows.Length == 0);
            DataRow newCD;
            int toPurchase = 0;

            // create a new row or use an existing one (when updating a quantity)
            if (!createNewRow)
            {
                newCD = existingProductRows[0];
                toPurchase = quantity;
            }
            else
            {
                newCD = cdPurchases.NewRow();
                toPurchase = quantity;
            }

            // insert all the CD values that we need for now
            newCD["CDID"]      = cdID;
            newCD["CDName"]    = cdName;
            newCD["quantity"]  = toPurchase;
            newCD["unitPrice"] = unitPrice;

            // add the new row to the table if it's a new row
            if (createNewRow)
                cdPurchases.Rows.Add(newCD);

            return true;
        }

        public bool IncrementCDsPurchased(int productID, int cdID, string cdName, int quantity, double unitPrice)
        {
            DataTable dtCDs = Common.GetCDShoppingCart(this.Session);
            int countSoFar  = quantity;
            DataRow[] results = dtCDs.Select("CDID = " + cdID);
            if (results.Length > 0)
            {
                countSoFar += (int)results[0]["quantity"];
            }

            return this.AddNewCD(productID, cdID, cdName, countSoFar, unitPrice);
        }

        /// <summary>
        /// Loads the shopping carts from the session and binds them to DataGrids.
        /// </summary>
        private void PopulateDataGrids()
        {
            DataTable bookPurchases = (DataTable) Session["BookPurchases"];
            if (bookPurchases != null && bookPurchases.Rows.Count > 0)
            {
                DataGridBooksInCart.DataSource = bookPurchases.DefaultView;
                DataGridBooksInCart.DataBind();
                DataGridBooksInCart.Visible = true;
                LabelShoppingCart.Visible = true;
                this.ButtonCheckout.Enabled = true;
            }
            else
            {
                DataGridBooksInCart.Visible = false;
            }

            DataTable musicPurchases = (DataTable) Session["CDPurchases"];
            if (musicPurchases != null && musicPurchases.Rows.Count > 0)
            {
                DataGridCDsInCart.DataSource = musicPurchases.DefaultView;
                DataGridCDsInCart.DataBind();
                DataGridCDsInCart.Visible = true;
                LabelShoppingCart.Visible = true;
                this.ButtonCheckout.Enabled = true;
            }
            else
            {
                DataGridCDsInCart.Visible = false;
            }
        }



		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
            this.Return.Click += new System.EventHandler(this.Return_Click);
            this.ButtonCheckout.Click += new System.EventHandler(this.ButtonCheckout_Click);
            this.Load += new System.EventHandler(this.Page_Load);

        }
		#endregion

		private void Return_Click(object sender, System.EventArgs e)
		{
			Response.Redirect("Inventory.aspx");
		}

		private void ButtonCheckout_Click(object sender, System.EventArgs e)
		{
			Response.Redirect("Checkout.aspx");
		}

        public void DataGridCDsInCart_Edit(Object sender, DataGridCommandEventArgs e)
        {
            this.DataGridCDsInCart.EditItemIndex = e.Item.ItemIndex;
            this.PopulateDataGrids();
        }

        public void DataGridCDsInCart_Update(Object sender, DataGridCommandEventArgs e)
        {
            // update some stuff
            DataTable CDPurchases = Common.GetCDShoppingCart(this.Session);
            string strQuantity = ((TextBox)e.Item.Cells[2].Controls[0]).Text.Trim();
            
            try
            {
                // update the new quantity accordingly
                int newQuantity = int.Parse(strQuantity);
                if (newQuantity > 0)
                {
                    DataRow cd = CDPurchases.Rows[e.Item.ItemIndex];
                    if (!this.AddNewCD(-1, int.Parse(cd["CDID"].ToString()), cd["CDName"].ToString(), newQuantity, double.Parse(cd["UnitPrice"].ToString())))
                    {
                        this.lblMusicError.Text = "Could not purchase " + newQuantity + " items.  There aren't that many in stock";
                        this.lblMusicError.Visible = true;
                    }
                }
                // delete the row if zero is specified
                else if (newQuantity == 0)
                {
                    CDPurchases.Rows[e.Item.ItemIndex].Delete();
                }
            }
            catch(Exception ex)
            {
                this.lblMusicError.Text = ex.Message;
                this.lblMusicError.Visible = true;
            }
            

            this.DataGridCDsInCart.EditItemIndex = -1;
            this.PopulateDataGrids();
        }

        public void DataGridCDsInCart_Cancel(Object sender, DataGridCommandEventArgs e)
        {
            this.DataGridCDsInCart.EditItemIndex = -1;
            this.PopulateDataGrids();
        }

        public void DataGridCDsInCart_Command(Object sender, DataGridCommandEventArgs e)
        {
            if (e.CommandName.ToUpper() == "DELETE")
            {
                DataTable CDPurchases = Common.GetCDShoppingCart(this.Session);
                CDPurchases.Rows.RemoveAt(e.Item.ItemIndex);
            }

            this.PopulateDataGrids();
        }







        public void DataGridBooksInCart_Edit(Object sender, DataGridCommandEventArgs e)
        {
            this.DataGridBooksInCart.EditItemIndex = e.Item.ItemIndex;
            this.PopulateDataGrids();
        }

        public void DataGridBooksInCart_Update(Object sender, DataGridCommandEventArgs e)
        {
            // update some stuff
            DataTable BookPurchases = Common.GetBookShoppingCart(this.Session);
            string strQuantity = ((TextBox)e.Item.Cells[2].Controls[0]).Text.Trim();
            
            try
            {
                // update the new quantity accordingly
                int newQuantity = int.Parse(strQuantity);
                if (newQuantity > 0)
                {
                    DataRow cd = BookPurchases.Rows[e.Item.ItemIndex];
                    if (!this.AddNewBooks(-1, int.Parse(cd["ISBN"].ToString()), cd["BookName"].ToString(), newQuantity, double.Parse(cd["UnitPrice"].ToString())))
                    {
                        this.lblBooksError.Text = "Could not purchase " + newQuantity + " items.  There aren't that many in stock";
                        this.lblBooksError.Visible = true;
                    }
                }
                    // delete the row if zero is specified
                else if (newQuantity == 0)
                {
                    BookPurchases.Rows[e.Item.ItemIndex].Delete();
                }
            }
            catch(Exception ex)
            {
                this.lblBooksError.Text = ex.Message;
                this.lblBooksError.Visible = true;
            }
            

            this.DataGridBooksInCart.EditItemIndex = -1;
            this.PopulateDataGrids();
        }

        public void DataGridBooksInCart_Cancel(Object sender, DataGridCommandEventArgs e)
        {
            this.DataGridBooksInCart.EditItemIndex = -1;
            this.PopulateDataGrids();
        }

        public void DataGridBooksInCart_Command(Object sender, DataGridCommandEventArgs e)
        {
            if (e.CommandName.ToUpper() == "DELETE")
            {
                DataTable BookPurchases = Common.GetBookShoppingCart(this.Session);
                BookPurchases.Rows.RemoveAt(e.Item.ItemIndex);
            }

            this.PopulateDataGrids();
        }

	}
}
