using System;
using System.Threading;
using System.Data;

namespace SqlRequest
{
    //represents multiple queries, either across multiple db's or within the same one
    //attempts at parallel querying and combination of results into a single dataset
    public class SQLBatch
    {
        #region Private Data

        private SQLRequest[] Queries;       
        private ManualResetEvent[] Events;
        private Relation[] Relations;
        private DataSet Result;

        #endregion

        #region Constructor

        public SQLBatch(params SQLRequest[] q)
        {
            this.Queries = new SQLRequest[q.Length];

            q.CopyTo(this.Queries,0L);

            this.Events = new ManualResetEvent[q.Length];
            this.Result = new DataSet("QueryResults");

            //initialize the elements needed internally to handle thread synchronization
            for(int x = 0 ; x < q.Length ; x++)
                this.Events[x] = new ManualResetEvent(false);
        }
        
      
        #endregion

        #region Public Methods

        //invoke all the queries, in serial
        public void RunQueries()
        {
            for(int x = 0 ; x < this.Queries.Length ; x++)
                ThreadPool.QueueUserWorkItem(new WaitCallback(ThreadFunc), new SQLPackage(this.Queries[x], this.Result, this.Events[x]));

            Thread t = new Thread(new ThreadStart(ThreadSync));
            t.ApartmentState = ApartmentState.MTA;

            //synchronize with all threads, need to wait on thread with MTA state because the main thread
            //seems to be STA and won't allow a wait all since it uses message loop to pass event messages
            //I think you can tell an app which threading model to use but I couldn't find where to do it
            t.Start();
            t.Join();
            
            ApplyRelations();           
        }

 
        public void SetRelations(params Relation[] r)
        {
            this.Relations = new Relation[r.Length];

            r.CopyTo(this.Relations,0L);
        }

        #endregion

        #region Private Methods
        
        private void ApplyRelations()
        {
            if(this.Relations == null)
                return;

            for(int x = 0 ; x < this.Relations.Length ; x++)
            {
                this.Relations[x].ApplyRelation(this.Result);
            }
        }


        private void ThreadSync()
        {
            //wait until all spawned threads have completed
            WaitHandle.WaitAll(this.Events);
        }

        //each request gets its own thread, who is passed in a thread package that encapsulates
        //the SQL query they want to issue, a place to put the result and also a ManualResetEvent
        //to be used internally for thread synchronization
        private void ThreadFunc(object o)
        {
            SQLPackage p = o as SQLPackage;

            DataSet t = p.Request.Invoke();

            //copy all the results into the single result DataSet
            //this is probably un-necessary because all results should just have one
            //resultant table, but bad to assume this ;)
            for(int x = 0 ; x < t.Tables.Count ; x++)
            {
                if(p.Request.ResultName != null)
                    t.Tables[x].TableName = p.Request.ResultName;
                else
                    t.Tables[x].TableName = "Table" + t.Tables[x].GetHashCode().ToString();


                //make sure to synchronize access on this object, since we all have the same Result object
                lock(p.Result)
                {
                    p.Result.Tables.Add(t.Tables[x].Copy());
                }
            }
            
            p.Event.Set();
        }
        
        #endregion

        #region Public Attributes


        public DataSet Results
        {
            get
            {
                return this.Result;
            }
        }

  
        #endregion
    }



    //internal class used to package up an SQLRequest, a destination DataSet and a ManualResetEvent for thread sync
    internal class SQLPackage
    {
        #region Private Data

        private SQLRequest r;
        private ManualResetEvent e;
        private DataSet d;

        #endregion

        #region Constructor

        public SQLPackage(SQLRequest sr, DataSet ds, ManualResetEvent ev)
        {
            this.r = sr;
            this.e = ev;
            this.d = ds;
        }

        #endregion

        #region Public Attributes

        public SQLRequest Request
        {
            get
            {
                return this.r;
            }
        }

        public ManualResetEvent Event
        {
            get
            {
                return this.e;
            }
        }

        public DataSet Result
        {
            get
            {
                return this.d;
            }            
        }

        #endregion

    }
}
