using System;
using System.Collections;
using System.Data.SqlClient;

namespace gibble
{
	/// <summary>
	/// Each package consists of PackageItems which are catalog numbers
	/// for each kind of item in a package, and the quantity of that item.
	/// </summary>
	public class PackageItem
	{
		private string item;
		private int quantity;

		public PackageItem(string item, int quantity)
		{
			this.item = item;
			this.quantity = quantity;
		}

		/// <summary>
		/// Gets or sets the catalog number of the item
		/// </summary>
		public string Item
		{
			get
			{
				return this.item;
			}
			set
			{
				this.item = value;
			}
		}

		/// <summary>
		/// Gets or sets the quantity of the item
		/// </summary>
		public int Quantity
		{
			get
			{
				return this.quantity;
			}
			set
			{
				this.quantity = value;
			}
		}

		/// <summary>
		/// Gets the weight of the package item
		/// </summary>
		/// <param name="connection">an open sql connection</param>
		/// <returns>quantity * product weight</returns>
		public int Weight(SqlConnection connection)
		{
			string sqlText =
				"SELECT weight " +
				"FROM frewen_inventory..Products " +
				"WHERE catalog# = @catalogNumber";
			SqlCommand cmd = new SqlCommand(sqlText, connection);
			cmd.Parameters.Add("@catalogNumber", this.Item);
			SqlDataReader reader = cmd.ExecuteReader();
			bool canRead = reader.Read();
			System.Diagnostics.Debug.Assert(canRead, "Package has an non existent product");
			int weight = reader.GetInt32(0) * this.Quantity;
			reader.Close();
			return weight;
		}
	}


	/// <summary>
	/// A Package object that will be shipped
	/// </summary>
	public class Package
	{
		
		private int warehouse;
		private int method;
		private ArrayList items;

		public Package(int warehouse)
		{
			this.warehouse = warehouse;
			this.items = new ArrayList();
			this.method = 0;
		}

		/// <summary>
		/// Adds a new item to the package
		/// </summary>
		/// <param name="item">the catalog number</param>
		/// <param name="quantity">the number of items to add</param>
		public void AddItem(string item, int quantity)
		{
			this.items.Add(new PackageItem(item, quantity));
		}

		/// <summary>
		/// Returns the weight of the package
		/// </summary>
		/// <param name="connection">an open sql connection</param>
		/// <returns>weight of the package</returns>
		public int Weight(SqlConnection connection)
		{
			int weight = 0;
			foreach (PackageItem i in this.items)
			{
				weight += i.Weight(connection);
			}
			return weight;
		}

		/// <summary>
		/// Returns the number of items in the package (accounts for quantity)
		/// </summary>
		/// <returns>the number of items in the package</returns>
		public int ItemCount()
		{
			int count = 0;
			foreach (PackageItem i in this.items)
			{
				count += i.Quantity;
			}
			return count;
		}

		/// <summary>
		/// The warehouse this package will ship from
		/// </summary>
		public int Warehouse
		{
			get
			{
				return this.warehouse;
			}
			set
			{
				this.warehouse = value;
			}
		}

		/// <summary>
		/// Gets or sets the shipping MethodID
		/// </summary>
		public int Method
		{
			get
			{
				return this.method;
			}
			set
			{
				this.method = value;
			}
		}

		/// <summary>
		/// Gets or sets the items
		/// </summary>
		public ArrayList Items
		{
			get
			{
				return this.items;
			}
			set
			{
				this.items = value;
			}
		}
	}
}
